#############################################################################################################################################################
#                                                                                                                                                           #
#UNIVERSIDADE FEDERAL FLUMINENSE - UFF                                                                                                                      #
#INSTITUTO DE COMPUTAÇÃO -                      DISCIPLINA: REDES DE COMPUTADORES 2017-2                                                                    #  
#                                                                                                                                                           #   
#                                   Exemplo 1 - Simulação de tráfego CBR (com 1.000 Bytes) em um circuito UDP com dois nós.                                 #  
#                                                   (N0 - Fonte  &&   N1 - Receptor)                                                                        #  
#                                                                                                                                                           #      
#############################################################################################################################################################


# Iniciando o simulador

set ns [new Simulator]

#Abrindo o arquivo NAM trace
 
set nf [open out.nam w]
$ns namtrace-all $nf

# Abrindo o arquivo de TRACE (Vazao, atraso, pacotes perdidos, recebitos e transmitidos) - O arquivo gerado é out-VAPkt.tr

set tf [open out-VAPkt.tr w]
$ns trace-all $tf


# Abrindo o arquivo de TRACE que será usado no Xgraph

set f0 [open udp-graph.tr w]



#Criando os dois nós do projeto

set n0 [$ns node]
set n1 [$ns node]

#Crinado um link duplex entre os nós. O DropTail significa que o primeiro a entrar e o primeiro a sair ou ser descartado se houver gargalo.

$ns duplex-link $n0 $n1 1Mb 10ms DropTail

#Criando um agente UDP e concetando ao nó N0

set udp0 [new Agent/UDP]
$ns attach-agent $n0 $udp0


#### CRIAR AQUI NESSE PONTO O TESTE DO BUFFER ######################################.


# Criando um tráfego CBR e concetando ao udp0

# portanto o tráfego gerado é = 8bits * 1000bytes/interval -  se a variável interval é 0.01s, teremos 8000/0,01 = 800k bits/s - menor que 1Mbit/s do canal, ou seja,
# sem perdas de pacotes.
# Porém, se tivermos a variável interval com 0,006, teremos 8000/0,006 = 1,33M bits/s - - maior que 1Mbit/s do canal, ou seja, com perdas de pacotes.

#Criando um agente Null agent (a traffic sink) e conectando ao nó N1


set cbr0 [new Application/Traffic/CBR]
$cbr0 set packetSize_ 1000  
$cbr0 set interval_ 0.0051  
$cbr0 attach-agent $udp0



#Criando um agente Null agent (a traffic sink) e conectando no nó N1. Importante passo para gerar o XGRAPH.

set null0 [new Agent/LossMonitor]
$ns attach-agent $n1 $null0

#Conectando 

$ns connect $udp0 $null0



# Definir procedimento que vai periodicamente gravar a largura de banda no tempo para gerar o gráfico.

proc record {} {
       global null0 f0
		
        
      set ns [Simulator instance]
        
     set time 0.1

#Quantos bytes tem sido recebidos pelo nó receptor N1?
		
      set bw0 [$null0 set bytes_]
        
        #Pegar o tempo corrente
		
        set now [$ns now]
	
        #Calcular a largura de banda (em MBit/s) e registrar no f0
		
       puts $f0 "$now [expr $bw0/$time*8/1000000]"
      
		
        #Resetar os valores de bytes no nó receptor N1
		
       $null0 set bytes_ 0
        
		
		
        #Re-agenda o procedimento
		
       $ns at [expr $now+$time] "record"
}



  
#Criando os intervalos de simulação

$ns at 0.0 "record"
$ns at 0.5 "$cbr0 start"
$ns at 4.5 "$cbr0 stop"
$ns at 5.5 "finish"




#Define um procedimento de fim para a simulação

proc finish {} {
        
	global ns nf tf f0
    $ns flush-trace
		
	#fechar o arquivo trace
	
    close $nf
	close $tf
	close $f0

	exec clear &	

	#Executar gawk para gerar informações de vazão, atrasos e pacotes retiradas do arquivo de TRACE VAPkt que serão mostradas na tela.

	exec gawk -f resultados-exemplo1.awk out-VAPkt.tr &
	
	exec clear &

	#Executar Xgraph no arquivo de TRACE. Visualizar o gráfico da vazão do UDP ao longo do tempo.	
	
	
	exec xgraph  out-xgraph.tr -geometry 700x400 &
	
	#Executar NAM no arquivo de TRACE
	
        exec nam out.nam &
        exit 0
}



#Rodar a simulação
$ns run





