#############################################################################################################################################################
#                                                                                                                                                           #
# UNIVERSIDADE FEDERAL FLUMINENSE - UFF                                                                                                                     #
# INSTITUTO DE COMPUTAÇÃO -                      DISCIPLINA: REDES DE COMPUTADORES 2017-2                                                                   #  
#                                                                                                                                                           #  
#                                                                                                                                                           # 
#       Exemplo2 - Simulação de tráfego CBR em um circuito UDP (N0 para N2) e tráfego FTP em um circuito TCP (N1 para N2) e gargalo N2 para N3 (final)      #  
#                                                                                                                                                           #  
#                                                                                                                                                           #      
#############################################################################################################################################################


# Iniciando o simulador

set ns [new Simulator]

#Define as cores para cada tráfego que será exibido pelo NAM. Azul é FTP e vermelho é UDP
$ns color 1 Blue  
$ns color 2 Red

#Abrindo o arquivo NAM trace
set nf [open out.nam w]
$ns namtrace-all $nf

#Abrindo um arquivo TRACE para analisar as medidas de todos os links
set tf [open out-all.tr w]
$ns trace-all $tf

#Abrindo arquivos TRACE para gerar um gráfico dos tráfegos UDP/CBR e TCP/FTP chegando no nó 3.

set f0 [open udp-n3.tr w]
set f1 [open tcp-n3.tr w]





#Define o procedimento de termino da simulação
proc finish {} {
        global ns nf tf tf12 tf02 tf23 f0 f1
        $ns flush-trace
	#fecha o trace do arquivo com todos os links juntos.
        close $tf
	#fecha o trace do arquivo do link12 (UDP/CBR)
        close $tf12
	#fecha o trace do link12 (TCP/FDP)
        close $tf02
	#fecha o trace do arquivo do link13 (UDP/CBR & TCP/FDP)
        close $tf23
	#fecha o trace do arquivo do grafico UDP
	close $f0
	#fecha o trace do arquivo do grafico TCP
	close $f1
    #fecha o arquivo de trace do NAM
        close $nf
	#Executa os dados trafegados  entre os nó 0 e o nó 2 (Tráfego TCP antes do gargalo)
        exec gawk -f resultados-exemplo2-link02.awk out-02.tr &
    #Executa os dados trafegados  entre os nó 1 e o nó 2 (Tráfego UDP antes do gargalo)
        exec gawk -f resultados-exemplo2-link12.awk out-12.tr &
	#Executa os dados trafegados  entre o nó 2 (gargalo) e o nó 3 (Tráfego UDP e TCP no gargalo)
        exec gawk -f resultados-exemplo2-link23.awk out-23.tr &
	#Executa os gráficos de dados trafegados  entre os nó 2 (gargalo) e o nó 3 (Tráfego UDP e TCP no gargalo)
        exec xgraph udp-n3.tr tcp-n3.tr -geometry 700x400 &

	#Executa o NAM para apresentar os resultados na tela dos passos da simulação
        exec nam out.nam &
        exit 0
}


#Criando os quatros nós da simulação

set n0 [$ns node]
set n1 [$ns node]
set n2 [$ns node]
set n3 [$ns node]

#Crinado os links entre os nós

$ns duplex-link $n0 $n2 2Mb 10ms DropTail
$ns duplex-link $n1 $n2 2Mb 10ms DropTail
$ns duplex-link $n2 $n3 1.7Mb 20ms DropTail

#Ajustando o tamanho máximo do bufer no gargalo nó 2.

$ns queue-limit $n2 $n3 10

#Posicionando os nós do projeto na tela do NAM
$ns duplex-link-op $n0 $n2 orient right-down
$ns duplex-link-op $n1 $n2 orient right-up
$ns duplex-link-op $n2 $n3 orient right

#Monitorando a fila no link (n2-n3). 
$ns duplex-link-op $n2 $n3 queuePos 0.5


#Monitorando a fila no link (n1-n2). 
$ns duplex-link-op $n1 $n2 queuePos 0.5

#Monitorando a fila no link (n0-n2). 
$ns duplex-link-op $n0 $n2 queuePos 0.5



#Ajustando a conecção TCP, nessa parte você pode alterar a forma do TCP operar, ou seja, se na linha  "set tcp [new Agent/TCP]" estiver ajustado desse modo teremos o TCP do tipo Tahoe, se a linha for
#alterada para set tcp [new Agent/TCP/Reno] teremos o segundo tipo de TCP que no caso é Reno e por último se a linha for alterada para set tcp [new Agent/TCP/Vegas] teremos o TCP Vegas operando na #simulação.

set tcp [new Agent/TCP]
$ns attach-agent $n0 $tcp
set sink [new Agent/TCPSink]
$ns attach-agent $n3 $sink
$ns connect $tcp $sink
$tcp set fid_ 1

#Definindo um FTP sobre a conecção TCP
set ftp [new Application/FTP]
$ftp attach-agent $tcp
$ftp set type_ FTP
#$ftp set packet_size_ 1000
#$ftp set interval_ 0.01 

#Ajustando a conecção UDP
set udp [new Agent/UDP]
$ns attach-agent $n1 $udp
set null [new Agent/LossMonitor]
$ns attach-agent $n3 $null
$ns connect $udp $null
$udp set fid_ 2

#Definindo um CBR sobre a conecção UDP
set cbr [new Application/Traffic/CBR]
$cbr attach-agent $udp
$cbr set type_ CBR
$cbr set packet_size_ 1000
$cbr set interval_ 0.0051 

# Definir procedimento que vai periodicamente gravar a largura de banda no tempo para gerar o gráfico.

proc record {} {
    
	global null f0 sink f1
		
        
    set ns [Simulator instance]
        
    set time 0.1

#Quantos bytes tem sido recebidos pelo nó receptor N3?
		
    set bw0 [$null set bytes_]

	set bw1 [$sink set bytes_]
        
    
#Pegar o tempo corrente
		
    set now [$ns now]

	
	
 #Calcular a largura de banda (em MBit/s) e registrar no f0
		
       
	puts $f0 "$now [expr $bw0/$time*8/1000000]"

	puts $f1 "$now [expr $bw1/$time*8/1000000]"
      
		
 #Resetar os valores de bytes no nó receptor N3
		
    $null set bytes_ 0
	
	$sink set bytes_ 0
        
		
		
  #Re-agenda o procedimento
		
    $ns at [expr $now+$time] "record"

	

}



#Criando os intervalos de simulação

$ns at 0.0 "record"
$ns at 0.1 "$ftp start"
$ns at 1.0 "$cbr start"
$ns at 5.0 "$cbr stop"
$ns at 7.0 "$ftp stop"


#Abrindo um arquivo TRACE para analisar as medidas do link (UDP/CBR) 1 -> 2
set tf12 [open out-12.tr w]
$ns at 0.1 "$ns trace-queue $n1 $n2 $tf12"


#Abrindo um arquivo TRACE para analisar as medidas do link (TCP/FDP) 0 -> 2
set tf02 [open out-02.tr w]
$ns at 0.1 "$ns trace-queue $n0 $n2 $tf02"

#Abrindo um arquivo TRACE para analisar as medidas do link (UDP/CBR & TCP/FDP) 2 -> 3
set tf23 [open out-23.tr w]
$ns at 0.1 "$ns trace-queue $n2 $n3 $tf23"


#Chama o fim da simulação 
$ns at 7.5 "finish"


#Rodar a simulação
$ns run
